SystemOrganization addCategory: #'Parsing-Derivatives-Test'!
SystemOrganization addCategory: #'Parsing-Derivatives'!
SystemOrganization addCategory: #'Parsing-Derivatives-Support'!
SystemOrganization addCategory: #'Parsing-Derivatives-Support-Tests'!

WeakValueDictionary subclass: #WeakDictionary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!WeakDictionary commentStamp: 'fbs 7/20/2012 22:17' prior: 0!
I am like a WeakValueDictionary, but I regard nil values as meaning "I don't have this thing".

That means that if a parser p is garbage collected, myself includesKey: p will return false. Thus, I look much like a normal Dictionary to my users.!

!WeakDictionary methodsFor: 'accessing' stamp: 'fbs 7/20/2012 19:00'!
at: key
	^ (super at: key)
		ifNil: [self errorKeyNotFound: key]! !

!WeakDictionary methodsFor: 'testing' stamp: 'fbs 7/24/2012 11:54'!
includesKey: anObject
	^ [(super includesKey: anObject) and: [(self at: anObject) notNil]]
		on: KeyNotFound do: [false]! !

!Collection methodsFor: '*Parsing-Derivatives' stamp: 'fbs 9/5/2012 09:25'!
cartesianProduct: aBinaryBlock with: aCollection
	"Given a collection A of a's and B of b's return a collection of all pairs of the form (a,b) where aBinaryBlock is responsible for creating the pairs."
	self ifEmpty: [^ self species new].
	self flag: #todo. "What about that 'as:' business? Seems inefficient to make an Array and then make a new collection from it, but how else to make this code polymorphic in the return type?"
	aCollection ifEmpty: [^ self species new].
	^ (self gather: [:a |
		aCollection collect: [:b |
			aBinaryBlock value: a value: b]]) as: self species.! !

Object subclass: #Compacter
	instanceVariableNames: 'cache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!Compacter commentStamp: 'fbs 9/10/2012 21:39' prior: 0!
I compact parsers. To that end, I walk the graph of a parser in a post-order fashion. At each step I map a parser to its compacted form. Since the walk is post-order, a composite parser (Cat, Union) will have already had its subparsers mapped.!

!Compacter class methodsFor: 'instance creation' stamp: 'fbs 9/10/2012 19:53'!
value: aParser
	^ self new value: aParser.! !

!Compacter methodsFor: 'private' stamp: 'fbs 9/17/2012 18:51'!
compact: aParser havingVisited: someParsers
	| visited |
	(someParsers includes: aParser)
		ifTrue: [aParser isDelegate
			ifTrue: [^ cache at: aParser ifAbsentPut: DelegateParser new]
			ifFalse: [^ cache at: aParser]].
	visited := someParsers copy.
	visited add: aParser.
	
	"Are we about to complete a cycle? If so, just move along."
	(aParser isDelegate and: [someParsers includes: aParser parser]) ifFalse: [
		aParser subParsers do: [:p | self compact: p havingVisited: visited]].
	^ cache
		at: aParser
		put: (aParser class
			caseOf: {
				[Cat] -> [self compactCat: aParser].
				[DelegateParser] -> [self compactDelegate: aParser].
				[Red] -> [self compactRed: aParser]}
			otherwise: [aParser])! !

!Compacter methodsFor: 'private' stamp: 'fbs 9/17/2012 09:31'!
compactCat: aCat
	| c1 c2 |
	cache at: aCat ifPresent: [:p | ^ p].
	c1 := cache at: aCat first.
	c2 := cache at: aCat second.
	c1 whenNullable: [:singleToken |
		^ c2 reduce: [:w2 |
			singleToken asOrderedCollection , w2]].
	c2 whenNullable: [:singleToken |
		^ c1 reduce: [:w1 |
			w1 asOrderedCollection , singleToken]].
	^ ((c1 == aCat first) and: [c2 == aCat second])
		ifTrue: [aCat]
		ifFalse: [cache at: aCat put: (c1 then: c2)]! !

!Compacter methodsFor: 'private' stamp: 'fbs 9/12/2012 19:13'!
compactDelegate: aDelegateParser
	"The compaction of a delegate parser is the compaction of its subparser. A delegate parser might not be resolvable yet, because if the delegate closes a cycle, the parser to which it should point will not have been mapped yet."
	| mappedSubparser |
	"This delegate completes a cycle. Its subparser hasn't been mapped yet. Mark its presence, and bail. The post-walk will resolve things."
	(cache includes: aDelegateParser parser) ifFalse: [^ DelegateParser new].
	mappedSubparser := cache at: aDelegateParser parser.
	(aDelegateParser parser == mappedSubparser) ifTrue: [^ aDelegateParser].
	(cache includesKey: aDelegateParser) ifTrue: [
		^ cache
			at: aDelegateParser
			put: (DelegateParser new parser: mappedSubparser)].
	
	^ cache at: aDelegateParser put: DelegateParser new.! !

!Compacter methodsFor: 'evaluating' stamp: 'fbs 9/18/2012 09:42'!
compactRed: aRed
	cache at: aRed ifPresent: [:p | ^ p].
	^ cache at: aRed parser
		ifPresent: [:compactedSubparser |
			(compactedSubparser = aRed parser)
				ifTrue: ["Already compact"
					aRed]
				ifFalse: [compactedSubparser reduce: aRed reducer]]
		ifAbsent: ["Very strange!! A post-order traversal should guarantee that the subparser's already compact!!"
			Error signal: 'Tried to compact a Red but its subparser is not yet compacted'].! !

!Compacter methodsFor: 'initialize-release' stamp: 'fbs 9/11/2012 09:44'!
initialize
	super initialize.
	cache := Dictionary new.! !

!Compacter methodsFor: 'private' stamp: 'fbs 9/12/2012 19:01'!
resolveDelegates: aParser
	PreOrderParserWalker
		walk: aParser
		doing: [:p |			
			(p isDelegate and: [p isResolved not])
				ifTrue: [| preimage |
					preimage := cache keyAtValue: p.
					p parser: (cache at: preimage parser)]].
	^ aParser! !

!Compacter methodsFor: 'evaluating' stamp: 'fbs 9/12/2012 09:29'!
value: aParser
	| visited |
	cache at: aParser ifPresent: [:compacted | ^ compacted].
	visited := IdentitySet new.
	^ self resolveDelegates: (self compact: aParser havingVisited: visited).! !

Object subclass: #Deriver
	instanceVariableNames: 'cache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!Deriver commentStamp: 'fbs 9/17/2012 09:36' prior: 0!
I take a parser and an object, and return the derivative of the parser with respect to the object.

That is, I return a parser that has consumed an object and computed the next step in the parser's progressive transformation from a parser into a parse forest.!

!Deriver methodsFor: 'evaluating' stamp: 'fbs 9/17/2012 09:43'!
d: anObject of: aParser
	| key |
	key := {aParser. anObject}.
	cache at: key ifPresent: [:derivedParser | ^ derivedParser].
	^ cache at: key put: (aParser d: anObject in: self).! !

!Deriver methodsFor: 'initialize-release' stamp: 'fbs 9/17/2012 09:16'!
initialize
	super initialize.
	cache := Dictionary new.! !

Object subclass: #DerivingParser
	instanceVariableNames: 'compacter'
	classVariableNames: 'EmptyParser EmptyStringParser'
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!DerivingParser commentStamp: 'fbs 9/4/2012 08:49' prior: 0!
My subclasses implement parsers that represent the _derivative_ of a context free language. Each time you "derive" a new parser with respect to a character you get a parser that
* builds a partial parse tree of the input thus far (the sequence of characters used to derive the current parser), and
* is a parser for the remaining input.

To put it another way, derivation calculates the parser's continuation.!

DerivingParser subclass: #Cat
	instanceVariableNames: 'first second'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!Cat class methodsFor: 'instance creation' stamp: 'fbs 8/30/2012 16:35'!
with: firstParser with: secondParser
	^ self new first: firstParser second: secondParser.! !

!Cat methodsFor: 'private' stamp: 'fbs 9/10/2012 19:52'!
compact: aCompacter
	| c1 c2 |
	self first whenNullable: [:singleToken |
		^ (aCompacter value: self second) reduce: [:w2 |
			singleToken asOrderedCollection , w2]].
	self second whenNullable: [:singleToken |
		^ (aCompacter value: self first) reduce: [:w1 |
			w1 asOrderedCollection , singleToken]].
	c1 := self first compact.
	c2 := self second compact.
	^ ((c1 == first) and: [c2 == second])
		ifTrue: [self]
		ifFalse: [c1 then: c2]! !

!Cat methodsFor: 'deriving' stamp: 'fbs 9/10/2012 19:19'!
d: anObject
	self first isNullable ifFalse: [^ (self first d: anObject) then: self second].
	
	^ ((EpsStar treeSet: self first parseNull) then: (self second d: anObject))
		or: ((self first d: anObject) then: self second)! !

!Cat methodsFor: 'deriving' stamp: 'fbs 9/17/2012 09:18'!
d: anObject in: aDeriver
	self first isNullable ifFalse: [^ (aDeriver d: anObject of: self first) then: self second].
	
	^ ((EpsStar treeSet: self first parseNull) then: (aDeriver d: anObject of: self second))
		or: ((aDeriver d: anObject of: self first) then: self second)! !

!Cat methodsFor: 'accessing' stamp: 'fbs 8/31/2012 08:03'!
first
	first isDelayed ifTrue: [first := first force].
	^ first.! !

!Cat methodsFor: 'initialize-release' stamp: 'fbs 8/30/2012 16:35'!
first: firstParser second: secondParser
	first := firstParser.
	second := secondParser.! !

!Cat methodsFor: 'testing' stamp: 'fbs 8/30/2012 16:36'!
isCat
	^ true.! !

!Cat methodsFor: 'printing' stamp: 'fbs 9/14/2012 21:06'!
printAsDotNode: parserIntegerMap
	^ ('{1}',
	'{2} -> {3} [label="first"]', Character cr,
	'{2} -> {4} [label="second"]', Character cr) format: {
		super printAsDotNode: parserIntegerMap.
		(parserIntegerMap at: self) printString.
		(parserIntegerMap at: self first) printString.
		(parserIntegerMap at: self second) printString.}! !

!Cat methodsFor: 'accessing' stamp: 'fbs 8/31/2012 08:03'!
second
	second isDelayed ifTrue: [second := second force].
	^ second.! !

!Cat methodsFor: 'accessing' stamp: 'fbs 9/5/2012 19:51'!
subParsers
	^ {self first. self second}.! !

DerivingParser subclass: #DelayedParser
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!DelayedParser commentStamp: 'fbs 9/6/2012 09:13' prior: 0!
I prevent a parser from being processed prematurely. I act much like a BlockClosure, in that I delay the evaluation of something until asked (via #force).

Parsers that use me often replace me with my forced parser, so my presence in a structure is usually temporary. That makes me look a bit like a Promise.!

!DelayedParser class methodsFor: 'instance creation' stamp: 'fbs 8/30/2012 21:14'!
on: aBlock
	^ self new block: aBlock! !

!DelayedParser methodsFor: 'initialize-release' stamp: 'fbs 8/30/2012 21:15'!
block: aBlock
	parser := aBlock.! !

!DelayedParser methodsFor: 'parsing' stamp: 'fbs 9/5/2012 09:47'!
compact
	^ self force compact.! !

!DelayedParser methodsFor: 'deriving' stamp: 'fbs 9/5/2012 09:47'!
d: anObject
	^ self force d: anObject.! !

!DelayedParser methodsFor: 'deriving' stamp: 'fbs 9/17/2012 09:18'!
d: anObject in: aDeriver
	^ aDeriver d: anObject of: self force.! !

!DelayedParser methodsFor: 'delaying' stamp: 'fbs 9/5/2012 09:40'!
delay
	^ self.! !

!DelayedParser methodsFor: 'printing' stamp: 'fbs 9/6/2012 09:41'!
dotLabel
	^ 'Delay'! !

!DelayedParser methodsFor: 'delaying' stamp: 'fbs 8/30/2012 21:30'!
force
	^ parser := parser value.! !

!DelayedParser methodsFor: 'testing' stamp: 'fbs 8/30/2012 21:15'!
isDelayed
	^ true.! !

!DelayedParser methodsFor: 'accessing' stamp: 'fbs 8/31/2012 08:09'!
parser
	parser isBlock ifTrue: [parser := parser value].
	^ parser.! !

!DelayedParser methodsFor: 'printing' stamp: 'fbs 9/14/2012 21:08'!
printAsDotNode: parserIntegerMap
	^ ('{1}',
	'{2} -> {3}', Character cr) format: {
		super printAsDotNode: parserIntegerMap.
		(parserIntegerMap at: self) printString.
		(parserIntegerMap at: self parser) printString.
	}.! !

!DelayedParser methodsFor: 'accessing' stamp: 'fbs 9/5/2012 19:41'!
subParsers
	^ {self force}.! !

DerivingParser subclass: #DelegateParser
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!DelegateParser commentStamp: 'fbs 8/31/2012 12:08' prior: 0!
I represent a reference to another parser. Use me to construct cycles in the parser structure:

	s := DelegateParser new.
	aStar := ($a asParser then: s) or: EmptyString new.
	s parser: aStar.!

!DelegateParser methodsFor: 'deriving' stamp: 'fbs 9/10/2012 19:20'!
d: anObject
	^ self parser d: anObject.! !

!DelegateParser methodsFor: 'deriving' stamp: 'fbs 9/17/2012 09:19'!
d: anObject in: aDeriver
	^ aDeriver d: anObject of: self parser.! !

!DelegateParser methodsFor: 'printing' stamp: 'fbs 9/6/2012 09:23'!
dotLabel
	^ 'Delegate'! !

!DelegateParser methodsFor: 'testing' stamp: 'fbs 9/11/2012 19:56'!
isDelegate
	^ true.! !

!DelegateParser methodsFor: 'testing' stamp: 'fbs 9/12/2012 09:33'!
isResolved
	^ parser notNil.! !

!DelegateParser methodsFor: 'accessing' stamp: 'fbs 8/31/2012 12:10'!
parser
	^ parser.! !

!DelegateParser methodsFor: 'accessing' stamp: 'fbs 8/31/2012 12:10'!
parser: aParser
	parser := aParser.! !

!DelegateParser methodsFor: 'printing' stamp: 'fbs 9/14/2012 21:09'!
printAsDotNode: parserIntegerMap
	^ ('{1}',
	'{2} -> {3}', Character cr) format: {
		super printAsDotNode: parserIntegerMap.
		(parserIntegerMap at: self) printString.
		(parserIntegerMap at: self parser) printString.
	}.! !

!DelegateParser methodsFor: 'accessing' stamp: 'fbs 9/12/2012 09:28'!
subParsers
	^ parser
		ifNil: [#()]
		ifNotNil: [{parser}].! !

!DelegateParser methodsFor: 'evaluating' stamp: 'fbs 9/12/2012 18:57'!
whenNullable: aUnaryBlock
	"Usually a DelegateParser will have a subparser. During compaction, the delegates in the compacted form might not be resolved because compaction hasn't completed. In that case, we treat the delegate as not being nullable (i.e., its parseNull collection isn't a single element)."
	self isResolved ifTrue:
		[super whenNullable: aUnaryBlock].! !

!DerivingParser class methodsFor: 'instance creation' stamp: 'fbs 8/30/2012 16:00'!
empty
	^ EmptyParser! !

!DerivingParser class methodsFor: 'instance creation' stamp: 'fbs 9/4/2012 08:56'!
emptyString
	^ EmptyStringParser! !

!DerivingParser class methodsFor: 'instance creation' stamp: 'fbs 9/4/2012 08:57'!
initialize
	EmptyParser := Empty new.
	EmptyStringParser := EmptyString new.! !

!DerivingParser methodsFor: 'parsing' stamp: 'fbs 9/18/2012 09:31'!
compact
	^ (compacter ifNil: [compacter := Compacter new]) value: self.! !

!DerivingParser methodsFor: 'parsing' stamp: 'fbs 9/10/2012 19:50'!
compacter
	^[:p | Compacter new value: p] memoized.! !

!DerivingParser methodsFor: 'deriving' stamp: 'fbs 8/30/2012 15:56'!
d: anObject
	self subclassResponsibility.! !

!DerivingParser methodsFor: 'deriving' stamp: 'fbs 9/17/2012 09:17'!
d: anObject in: aDeriver
	self subclassResponsibility.! !

!DerivingParser methodsFor: 'delaying' stamp: 'fbs 8/30/2012 21:29'!
delay
	^ [self] asParser.! !

!DerivingParser methodsFor: 'deriving' stamp: 'fbs 9/17/2012 09:23'!
derive: anObject
	^ Deriver new d: anObject of: self.! !

!DerivingParser methodsFor: 'printing' stamp: 'fbs 9/6/2012 09:16'!
dotLabel
	^ self className! !

!DerivingParser methodsFor: 'delaying' stamp: 'fbs 9/5/2012 09:49'!
force
	^ self.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 8/30/2012 16:35'!
isCat
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 8/30/2012 21:14'!
isDelayed
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 9/11/2012 19:55'!
isDelegate
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 9/18/2012 16:09'!
isEmpty
	"Is the language I define I the empty set?"
	| empty |
	empty := nil.
	empty := [:p |
		p class caseOf: {
			[Empty] -> [true].
			[EmptyString] -> [false].
			[EpsStar] -> [false].
			[Literal] -> [false].
			[StarParser] -> [false].
			[Union] -> [(empty value: p left) and: [empty value: p right]].
			[Cat] -> [(empty value: p first) or: [empty value: p second]].
			[Red] -> [empty value: p parser].
			[DelegateParser] -> [p isResolved ifTrue: [empty value: p parser] ifFalse: ["We can't say!!" false]].
			[Suture] -> [false "We can't say!!"].
			[DelayedParser] -> [empty value: p force]}
			otherwise: [Error signal: 'isEmpty not defined for ', p className]] lfpWithBottom: false.
	^ empty value: self.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 8/30/2012 15:59'!
isEps
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 9/4/2012 09:04'!
isEpsStar
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 8/30/2012 16:28'!
isLazy
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 8/30/2012 15:56'!
isLiteral
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 9/13/2012 19:48'!
isNullable
	"Can I parse the empty string?"
	| nullable |
	nullable := nil.
	nullable := [:p |
		p class caseOf: {
			[Empty] -> [false].
			[EmptyString] -> [true].
			[EpsStar] -> [true].
			[Literal] -> [false].
			[Union] -> [(nullable value: p left) or: [nullable value: p right]].
			[Cat] -> [(nullable value: p first) and: [nullable value: p second]].
			[Red] -> [nullable value: p parser].
			[StarParser] -> [(nullable value: p parser) or: [p parser isEmpty]].
			[DelayedParser] -> [nullable value: p force].
			[DelegateParser] -> [nullable value: p parser]}
		otherwise: [Error signal: 'isNullable not defined for ', p className]] lfpWithBottom: false.
	^ nullable value: self.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 8/30/2012 16:43'!
isRed
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 9/13/2012 19:35'!
isStar
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 8/30/2012 16:48'!
isUnion
	^ false.! !

!DerivingParser methodsFor: 'parsing' stamp: 'fbs 9/5/2012 09:54'!
or: aParser
	aParser isEmpty ifTrue: [^ self].
	^ Union left: self delay right: aParser delay.! !

!DerivingParser methodsFor: 'parsing' stamp: 'fbs 9/18/2012 15:45'!
parseNull
	"Can I parse the empty string?"
	| parseNull |
	parseNull := nil.
	parseNull := [:p |
		p class caseOf: {
			[Empty] -> [Set new].
			[EmptyString] -> [Set newFrom: #(#())].
			[EpsStar] -> [p treeSet].
			[Literal] -> [Set new].
			[Union] -> [(parseNull value: p left) union: (parseNull value: p right)].
			[Cat] -> [ | concat |
				concat := [:t1 :t2 | t1 asArray , t2 asArray].
				(parseNull value: p first)
					cartesianProduct: concat
					with: (parseNull value: p second)].
			[Red] -> [(parseNull value: p parser) collect: p reducer].
			[StarParser] -> [Set newFrom: #(#())].
			[DelayedParser] -> [parseNull value: p force].
			[DelegateParser] -> [p isResolved ifFalse: [Error signal: 'Unresolved delegate']. parseNull value: p parser]}
		otherwise: [Error signal: 'parseNull not defined for ', p className]] lfpWithBottom: Set new.
	^ parseNull value: self.! !

!DerivingParser methodsFor: 'printing' stamp: 'fbs 9/14/2012 21:18'!
printAsDigraph
	| c intmap i |
	"Post-order makes for nicer graphs."
	i := 0.
	intmap := Dictionary new.
	PreOrderParserWalker walk: self collecting: [:p | intmap at: p put: i. i := i + 1].
	c := (PostOrderParserWalker walk: self collecting: [:p | p printAsDotNode: intmap]) asOrderedCollection.
	c reverseInPlace.
	c addFirst: ('digraph {', Character cr).
	c addLast: ('}', Character cr).
	^ c reduce: #,.! !

!DerivingParser methodsFor: 'printing' stamp: 'fbs 9/14/2012 21:09'!
printAsDotNode: parserIntegerMap
	"parserIntegerMap maps parsers to integers. We do this manually because #identityHash is not a reliable way (by design) of distinguishing between two objects."
	^ self printNode: parserIntegerMap.! !

!DerivingParser methodsFor: 'printing' stamp: 'fbs 9/14/2012 21:20'!
printNode: parserIntegerMap
	^ ('{1} [label="{2} ({1})"]' format: {(parserIntegerMap at: self) printString. self dotLabel}),
		Character cr.! !

!DerivingParser methodsFor: 'parsing' stamp: 'fbs 8/30/2012 16:42'!
reduce: aUnaryBlock
	^ Red parser: self reduceWith: aUnaryBlock! !

!DerivingParser methodsFor: 'parsing' stamp: 'fbs 9/13/2012 19:36'!
star
	^ StarParser repeating: self.! !

!DerivingParser methodsFor: 'accessing' stamp: 'fbs 9/5/2012 19:34'!
subParsers
	"Most parsers don't have subparsers."
	^ #().! !

!DerivingParser methodsFor: 'parsing' stamp: 'fbs 9/5/2012 09:54'!
then: aParser
	aParser isEmpty ifTrue: [^ DerivingParser empty].
	^ Cat with: self delay with: aParser delay.! !

!DerivingParser methodsFor: 'evaluating' stamp: 'fbs 9/10/2012 21:18'!
whenNullable: aUnaryBlock
	| trees |
	trees := self parseNull.
	(trees size = 1) ifTrue: [aUnaryBlock value: trees anyOne].! !

DerivingParser subclass: #Empty
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!Empty commentStamp: 'fbs 8/30/2012 16:09' prior: 0!
I represent the empty parser.!

!Empty methodsFor: 'deriving' stamp: 'fbs 8/30/2012 16:09'!
d: anObject
	^ self.! !

!Empty methodsFor: 'deriving' stamp: 'fbs 9/17/2012 09:20'!
d: anObject in: aDeriver
	^ self.! !

!Empty methodsFor: 'parsing' stamp: 'fbs 8/30/2012 18:17'!
or: aParser
	^ aParser.! !

!Empty methodsFor: 'parsing' stamp: 'fbs 8/30/2012 18:03'!
then: aParser
	^ self.! !

DerivingParser subclass: #EmptyString
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!EmptyString commentStamp: 'fbs 8/30/2012 16:23' prior: 0!
I represent a parser that may parse the empty string.!

!EmptyString methodsFor: 'deriving' stamp: 'fbs 9/4/2012 09:00'!
d: anObject
	^ DerivingParser empty.! !

!EmptyString methodsFor: 'deriving' stamp: 'fbs 9/17/2012 09:20'!
d: anObject in: aDeriver
	^ DerivingParser empty.! !

!EmptyString methodsFor: 'enumerating' stamp: 'fbs 9/5/2012 19:07'!
do: aBlock
	"Simulate an empty collection."! !

!EmptyString methodsFor: 'printing' stamp: 'fbs 9/6/2012 09:26'!
dotLabel
	^ '{1} ({2})' format: {'eps'. self identityHash printString}.! !

!EmptyString methodsFor: 'testing' stamp: 'fbs 9/4/2012 09:07'!
isEps
	^ true.! !

!EmptyString methodsFor: 'printing' stamp: 'fbs 9/18/2012 16:00'!
printAsDotNode: parserIntegerMap
	^ ('{1} [label="{2} ({1})"]' format: {
		(parserIntegerMap at: self) printString.
		self dotLabel}),
	Character cr.! !

EmptyString subclass: #EpsStar
	instanceVariableNames: 'treeSet'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!EpsStar commentStamp: 'fbs 8/30/2012 16:10' prior: 0!
I represent an intermediate state in the parsing of some input. treeSet forms the collective seed of a set of parse trees.!

!EpsStar class methodsFor: 'instance creation' stamp: 'fbs 8/30/2012 15:58'!
treeSet: aCollection
	^ self new treeSet: aCollection asSet.! !

!EpsStar methodsFor: 'deriving' stamp: 'fbs 8/30/2012 16:13'!
d: anObject
	^ self.! !

!EpsStar methodsFor: 'deriving' stamp: 'fbs 9/17/2012 09:20'!
d: anObject in: aDeriver
	^ self.! !

!EpsStar methodsFor: 'testing' stamp: 'fbs 9/4/2012 09:05'!
isEps
	"This is rubbish. We should actually have the eps and eps* be subclasses of a common ancestor."
	self flag: #todo.
	^ false.! !

!EpsStar methodsFor: 'testing' stamp: 'fbs 9/4/2012 09:04'!
isEpsStar
	^ true.! !

!EpsStar methodsFor: 'printing' stamp: 'fbs 9/14/2012 21:12'!
printAsDotNode: parserIntegerMap
	^ ('{1} [label="eps* ({2} ({3})"]' format: {
		(parserIntegerMap at: self) printString.
		self treeSet printString.
		(parserIntegerMap at: self) printString}),
	Character cr.! !

!EpsStar methodsFor: 'accessing' stamp: 'fbs 8/30/2012 15:59'!
treeSet
	^ treeSet! !

!EpsStar methodsFor: 'initialize-release' stamp: 'fbs 8/30/2012 15:58'!
treeSet: aCollection
	treeSet := aCollection.! !

DerivingParser subclass: #Literal
	instanceVariableNames: 'token'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!Literal commentStamp: 'fbs 8/30/2012 16:10' prior: 0!
I represent a parser that accepts some literal. Note that parsing with derivatives is not limited to parsing streams of characters, so my literal could be any object (not already parsed by other parsers).!

!Literal class methodsFor: 'instance creation' stamp: 'fbs 8/30/2012 15:55'!
token: anObject
	^ self new token: anObject! !

!Literal methodsFor: 'deriving' stamp: 'fbs 8/30/2012 16:23'!
d: anObject
	^ (token = anObject)
		ifTrue: [EpsStar treeSet: {token}]
		ifFalse: [DerivingParser empty].! !

!Literal methodsFor: 'deriving' stamp: 'fbs 9/17/2012 09:21'!
d: anObject in: aDeriver
	^ (token = anObject)
		ifTrue: [EpsStar treeSet: {token}]
		ifFalse: [DerivingParser empty].! !

!Literal methodsFor: 'testing' stamp: 'fbs 8/30/2012 15:56'!
isLiteral
	^ true.! !

!Literal methodsFor: 'printing' stamp: 'fbs 9/17/2012 18:54'!
printAsDotNode: parserIntegerMap
	^ ('{1} [label="Literal ({2}) ({1})"]' format: {
		(parserIntegerMap at: self) printString.
		self token printString}),
	Character cr.! !

!Literal methodsFor: 'accessing' stamp: 'fbs 9/5/2012 10:00'!
token
	^ token.! !

!Literal methodsFor: 'initialize-release' stamp: 'fbs 8/30/2012 15:55'!
token: anObject
	token := anObject! !

DerivingParser subclass: #Red
	instanceVariableNames: 'parser reducer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!Red commentStamp: 'fbs 8/30/2012 20:46' prior: 0!
I represent a "reducing" parser: a parser that applies a certain function to every element of my subparser's partial parse trees.

I am the parser that allows you to express your semantic actions (such as building ASTs).!

!Red class methodsFor: 'instance creation' stamp: 'fbs 8/30/2012 16:45'!
parser: aParser reduceWith: aUnaryBlock
	^ self new parser: aParser reduceWith: aUnaryBlock.! !

!Red methodsFor: 'deriving' stamp: 'fbs 8/30/2012 20:39'!
d: anObject
	^ (parser d: anObject) reduce: reducer.! !

!Red methodsFor: 'deriving' stamp: 'fbs 9/17/2012 09:21'!
d: anObject in: aDeriver
	^ (aDeriver d: anObject of: parser) reduce: reducer.! !

!Red methodsFor: 'printing' stamp: 'fbs 9/6/2012 09:27'!
dotLabel
	^ '{1} (add {2})' format: {self className. [(self reduce at: 1) printString] on: Error do: ['??']}! !

!Red methodsFor: 'testing' stamp: 'fbs 8/30/2012 16:45'!
isRed
	^ true.! !

!Red methodsFor: 'accessing' stamp: 'fbs 8/30/2012 22:09'!
parser
	^ parser.! !

!Red methodsFor: 'initialize-release' stamp: 'fbs 8/30/2012 16:45'!
parser: aParser reduceWith: aUnaryBlock
	parser := aParser.
	reducer := aUnaryBlock.! !

!Red methodsFor: 'printing' stamp: 'fbs 9/14/2012 21:14'!
printAsDotNode: parserIntegerMap
	^ ('{1}',
	'{2} -> {3}', Character cr) format: {
		super printAsDotNode: parserIntegerMap.
		(parserIntegerMap at: self) printString.
		(parserIntegerMap at: self parser) printString.
	}.! !

!Red methodsFor: 'parsing' stamp: 'fbs 9/18/2012 09:41'!
reduce: aUnaryBlock
	^ Red parser: self parser reduceWith: (aUnaryBlock compose: self reducer)! !

!Red methodsFor: 'accessing' stamp: 'fbs 8/30/2012 20:40'!
reducer
	^ reducer.! !

!Red methodsFor: 'accessing' stamp: 'fbs 9/5/2012 19:35'!
subParsers
	^ {parser}.! !

DerivingParser subclass: #StarParser
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!StarParser class methodsFor: 'instance creation' stamp: 'fbs 9/13/2012 19:36'!
repeating: aParser
	^ self new parser: aParser.! !

!StarParser methodsFor: 'deriving' stamp: 'fbs 9/13/2012 20:42'!
d: anObject
	^ (parser d: anObject) then: self.! !

!StarParser methodsFor: 'deriving' stamp: 'fbs 9/17/2012 09:22'!
d: anObject in: aDeriver
	^ (aDeriver d: anObject of: parser) then: self.! !

!StarParser methodsFor: 'testing' stamp: 'fbs 9/13/2012 19:37'!
isStar
	^ true.! !

!StarParser methodsFor: 'accessing' stamp: 'fbs 9/13/2012 19:49'!
parser
	^ parser.! !

!StarParser methodsFor: 'initialize-release' stamp: 'fbs 9/13/2012 19:37'!
parser: aParser
	parser := aParser.! !

!StarParser methodsFor: 'printing' stamp: 'fbs 9/14/2012 21:09'!
printAsDotNode: parserIntegerMap
	^ ('{1}',
	'{2} -> {3}', Character cr) format: {
		super printAsDotNode: parserIntegerMap.
		(parserIntegerMap at: self) printString.
		(parserIntegerMap at: self parser) printString.
	}.! !

!StarParser methodsFor: 'accessing' stamp: 'fbs 9/14/2012 20:48'!
subParsers
	^ {parser}.! !

DerivingParser subclass: #Union
	instanceVariableNames: 'left right'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!Union class methodsFor: 'instance creation' stamp: 'fbs 8/30/2012 16:50'!
left: lParser right: rParser
	^ self new left: lParser right: rParser.! !

!Union methodsFor: 'deriving' stamp: 'fbs 8/30/2012 21:11'!
d: anObject
	^ [left d: anObject] asParser or: [right d: anObject] asParser.! !

!Union methodsFor: 'deriving' stamp: 'fbs 9/17/2012 09:22'!
d: anObject in: aDeriver
	^ [aDeriver d: anObject of: left] asParser or: [aDeriver d: anObject of: right] asParser.! !

!Union methodsFor: 'testing' stamp: 'fbs 8/30/2012 16:52'!
isUnion
	^ true.! !

!Union methodsFor: 'accessing' stamp: 'fbs 8/31/2012 08:47'!
left
	left isDelayed ifTrue: [left := left force].
	^ left.! !

!Union methodsFor: 'initialize-release' stamp: 'fbs 8/30/2012 16:51'!
left: lParser right: rParser
	left := lParser.
	right := rParser.! !

!Union methodsFor: 'printing' stamp: 'fbs 9/14/2012 21:06'!
printAsDotNode: parserIntegerMap
	^ ('{1}',
	'{2} -> {3} [label="left"]', Character cr,
	'{2} -> {4} [label="right"]', Character cr) format: {
		super printAsDotNode: parserIntegerMap.
		(parserIntegerMap at: self) printString.
		(parserIntegerMap at: self left) printString.
		(parserIntegerMap at: self right) printString.}! !

!Union methodsFor: 'accessing' stamp: 'fbs 8/31/2012 08:47'!
right
	right isDelayed ifTrue: [right := right force].
	^ right.! !

!Union methodsFor: 'accessing' stamp: 'fbs 9/5/2012 19:51'!
subParsers
	^ {self left. self right.}.! !

Object subclass: #LeastFixedPoint
	instanceVariableNames: 'changed running visited cache bottom block'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!LeastFixedPoint commentStamp: 'fbs 7/13/2012 18:48' prior: 0!
I calculate the least fixed point of some calculation. That is, given a calculation f and an initial value x, I repeatedly calculate x := f value: x until x == f value x.

My implementation comes from a translation of Matt Might's derp library, from his "YACC is Dead" work.!

!LeastFixedPoint class methodsFor: 'instance creation' stamp: 'fbs 7/13/2012 09:45'!
run: aBlock bottom: anObject
	^ self new run: aBlock bottom: anObject.! !

!LeastFixedPoint methodsFor: 'initialize-release' stamp: 'fbs 7/20/2012 19:00'!
run: aBlock bottom: anObject
	self initialize.
	cache := WeakDictionary new.
	changed := DelimitedDynamicVariable default: #errorChanged.
	running := DelimitedDynamicVariable default: false.
	visited := DelimitedDynamicVariable default: #errorVisited.
	block := aBlock.
	bottom := anObject.! !

!LeastFixedPoint methodsFor: 'evaluating' stamp: 'fbs 8/7/2012 07:56'!
value: anObject
	| cached cachedValue run |
	cached := cache includesKey: anObject.
	cachedValue := cache at: anObject ifAbsent: [bottom].
	run := running dref.
	
	(cached and: [run not]) ifTrue: [^ cachedValue].
	(run and: [visited dref includesKey: anObject]) ifTrue:
		[^ cached ifTrue: [cachedValue] ifFalse: [bottom]].
	
	run ifTrue: [ | newVal |
		visited dref at: anObject put: true.
		newVal := block value: anObject.
		newVal ~= cachedValue ifTrue: [
			changed dset: true.
			cache at: anObject put: newVal].
		^ newVal].
	
	"Technically this condition's not necessary. It causes all the cases to be non-overlapping."
	(cached not and: [run not]) ifTrue: [
		changed, running, visited dlet: {true. true. WeakDictionary new} in: [ | v |
			v := anObject.
			[changed dref] whileTrue: [
				changed dset: false.
				visited dset: WeakDictionary new.
				v := self value: anObject].
			^ v]].
	
	Error signal: 'It''s impossible to reach here.'.! !

Object subclass: #MemoizingBlock
	instanceVariableNames: 'block cache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!MemoizingBlock commentStamp: 'fbs 7/17/2012 13:50' prior: 0!
A MemoizingBlock is a block-like thing that remembers the results of its previous invocations. On repeated invocations, it returns this stored result rather than re-executing.!

!MemoizingBlock class methodsFor: 'as yet unclassified' stamp: 'fbs 7/17/2012 13:53'!
run: aBlock
	^ self new run: aBlock! !

!MemoizingBlock methodsFor: 'initialize-release' stamp: 'fbs 8/14/2012 22:36'!
initialize
	cache := Dictionary new.! !

!MemoizingBlock methodsFor: 'initialize-release' stamp: 'fbs 8/2/2012 09:26'!
run: aBlock
	block := aBlock.! !

!MemoizingBlock methodsFor: 'evaluating' stamp: 'fbs 7/17/2012 20:15'!
value: anObject
	^ self valueWithArguments: {anObject}.! !

!MemoizingBlock methodsFor: 'evaluating' stamp: 'fbs 7/17/2012 20:15'!
value: anObject value: anotherObject
	^ self valueWithArguments: {anObject. anotherObject}.! !

!MemoizingBlock methodsFor: 'evaluating' stamp: 'fbs 7/17/2012 20:24'!
value: objectA value: objectB value: objectC
	^ self valueWithArguments: {objectA. objectB. objectC}.! !

!MemoizingBlock methodsFor: 'evaluating' stamp: 'fbs 7/17/2012 20:24'!
value: objectA value: objectB value: objectC value: objectD
	^ self valueWithArguments: {objectA. objectB. objectC. objectD}.! !

!MemoizingBlock methodsFor: 'evaluating' stamp: 'fbs 7/17/2012 20:15'!
valueWithArguments: someObjects
	^ cache at: someObjects ifAbsentPut: [block valueWithArguments: someObjects].! !

!Object methodsFor: '*Parsing-Derivatives' stamp: 'fbs 9/14/2012 09:33'!
asArray
	^ {self}.! !

!Object methodsFor: '*Parsing-Derivatives' stamp: 'fbs 8/30/2012 16:07'!
asParser
	^ Literal token: self.! !

Object subclass: #ParserWalker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!ParserWalker class methodsFor: 'enumerating' stamp: 'fbs 9/12/2012 09:31'!
walk: aParser collecting: aUnaryBlock
	"Walk all parsers reachable from aParser, executing aUnaryBlock on each parser and returning the collection of results (in traversal order). By default ParserWalker will use a cycle-safe pre-order traversal. Invoke #walk:collecting: on subclasses for other traversals."
	^ ((self == ParserWalker ifTrue: [PreOrderParserWalker] ifFalse: [self])
		walk: aParser
		collectingMap: aUnaryBlock)
			pairsCollect: [:a :b | b].! !

!ParserWalker class methodsFor: 'private' stamp: 'fbs 9/8/2012 21:43'!
walk: aParser collecting: aUnaryBlock withCache: someParsers andMap: someAssocs
	self subclassResponsibility.! !

!ParserWalker class methodsFor: 'enumerating' stamp: 'fbs 9/8/2012 23:15'!
walk: aParser collectingMap: aUnaryBlock
	| cache mappedValues |
	"mappedValues contains a plist: {key1. value1. key2. value2. ...}. We don't use a Dictionary because we might care about the order in which we walk the graph, and hence the order of mapped values. cache contains all visited parsers, to render the traversal cycle-safe."
	cache := IdentitySet new.
	mappedValues := OrderedCollection new.
	^ (self walk: aParser collecting: aUnaryBlock withCache: cache andMap: mappedValues) second.! !

!ParserWalker class methodsFor: 'enumerating' stamp: 'fbs 9/12/2012 09:31'!
walk: aParser doing: aUnaryBlock
	"Walk all parsers reachable from aParser, executing aUnaryBlock on each parser. By default ParserWalker will use a cycle-safe pre-order traversal. Invoke #walk:doing: on subclasses for other traversals."
	((self == ParserWalker ifTrue: [PreOrderParserWalker] ifFalse: [self])
		walk: aParser
		collectingMap: aUnaryBlock)
			pairsCollect: [:a :b | b].
	^ self.! !

ParserWalker subclass: #PostOrderParserWalker
	instanceVariableNames: ''
	classVariableNames: 'NotYetResolvedMarker'
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!PostOrderParserWalker class methodsFor: 'class initialization' stamp: 'fbs 9/8/2012 12:35'!
initialize
	"My presence in a cache represents that (a) this node has been visited but (b) its mapped value has not been calculated (and will be after tree traversal unwinds)."
	NotYetResolvedMarker := Object new.! !

!PostOrderParserWalker class methodsFor: 'private' stamp: 'fbs 9/8/2012 23:21'!
walk: aParser collecting: aUnaryBlock withCache: someParsers andMap: someAssocs
	| newCache newMap |
	(someParsers includes: aParser) ifTrue: [^ {someParsers. someAssocs}].
	
	newMap := someAssocs copy.
	newCache := someParsers copy.
	newCache add: aParser.
	aParser subParsers collect: [:p | | result |
		result := self walk: p collecting: aUnaryBlock withCache: newCache andMap: newMap.
		newCache addAll: result first.
		result second pairsDo: [:a :b |
			(newMap includes: a) ifFalse: [newMap addAll: {a. b}]]].
	newMap addAll: {aParser. aUnaryBlock value: aParser}.
	^ {newCache. newMap}.! !

ParserWalker subclass: #PreOrderParserWalker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!PreOrderParserWalker class methodsFor: 'private' stamp: 'fbs 9/8/2012 23:18'!
walk: aParser collecting: aUnaryBlock withCache: someParsers andMap: someAssocs
	| newCache newMap |
	(someParsers includes: aParser) ifTrue: [^ {someParsers. someAssocs}].
	
	newMap := someAssocs copy.
	newCache := someParsers copy.
	newCache add: aParser.
	newMap addAll: {aParser. aUnaryBlock value: aParser}.
	aParser subParsers collect: [:p | | result |
		result := self walk: p collecting: aUnaryBlock withCache: newCache andMap: newMap.
		newCache addAll: result first.
		result second pairsDo: [:a :b |
			(newMap includes: a) ifFalse: [newMap addAll: {a. b}]]].
	^ {newCache. newMap}.! !

Error subclass: #ArityMismatch
	instanceVariableNames: 'expectedArgCount actualArgCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!ArityMismatch commentStamp: 'fbs 7/20/2012 22:01' prior: 0!
I represent someone trying to invoke a block with the incorrect number of arguments.!

!ArityMismatch class methodsFor: 'as yet unclassified' stamp: 'fbs 7/20/2012 22:03'!
actual: actual expected: expected
	^ self new initializeWithActualCount: actual expectedCount: expected.! !

!ArityMismatch methodsFor: 'accessing' stamp: 'fbs 7/20/2012 22:02'!
actualArgCount
	^ actualArgCount.! !

!ArityMismatch methodsFor: 'accessing' stamp: 'fbs 7/20/2012 22:02'!
expectedArgCount
	^ expectedArgCount.! !

!ArityMismatch methodsFor: 'initialize-release' stamp: 'fbs 7/20/2012 22:02'!
initializeWithActualCount: actual expectedCount: expected
	actualArgCount := actual.
	expectedArgCount := expected.! !

!BlockClosure methodsFor: '*Parsing-Derivatives-Support' stamp: 'fbs 7/18/2012 18:55'!
arity
	^ self numArgs.! !

!BlockClosure methodsFor: '*Parsing-Derivatives-Support' stamp: 'fbs 8/30/2012 21:12'!
asParser
	^ DelayedParser on: self.! !

!BlockClosure methodsFor: '*Parsing-Derivatives-Support' stamp: 'fbs 7/20/2012 22:04'!
lfpWithBottom: anObject
	"Turn self (a unary BlockClosure) into a recursive (yet monotonic) function over a mutually recursive graph by computing its (least) fixed point."
	self numArgs = 1 ifFalse: [(ArityMismatch actual: self numArgs expected: 1) signal].
	^ LeastFixedPoint run: self bottom: anObject! !

!BlockClosure methodsFor: '*Parsing-Derivatives-Support' stamp: 'fbs 7/17/2012 18:45'!
memoized
	^ MemoizingBlock run: self.! !

!Symbol methodsFor: '*Parsing-Derivatives-Support' stamp: 'fbs 7/18/2012 18:55'!
arity
	"A Symbol without a : takes one argument - the receiver."
	^ self numArgs + 1.! !

!Symbol methodsFor: '*Parsing-Derivatives-Support' stamp: 'fbs 7/20/2012 22:07'!
lfpWithBottom: anObject
	(self arity = 1) ifFalse: [(ArityMismatch actual: self arity expected: 1) signal].
	^ [:x | self value: x] lfpWithBottom: anObject.! !

TestCase subclass: #CompacterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!CompacterTest methodsFor: 'testing - recursive grammars' stamp: 'fbs 9/13/2012 19:32'!
testCycleStartingWithDelegateCanCompact
	| ones d Dones|
	d := DelegateParser new.
	ones := DerivingParser emptyString or: (1 asParser then: d).
	d parser: ones.
	Dones := ones d: 1.
	"(a) It can actually compact. (b) The compacted form still defines the same language."
	self assert: Dones parseNull equals: (Compacter value: Dones) parseNull.! !

!CompacterTest methodsFor: 'testing - recursive grammars' stamp: 'fbs 9/10/2012 21:27'!
testRightRecursiveGrammarsCanCompact
	| ones d |
	d := DelegateParser new.
	ones := DerivingParser emptyString or: (1 asParser then: d).
	d parser: ones.
	self assert: ones equals: (Compacter value: ones).! !

TestCase subclass: #DerivingParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!DerivingParserTest methodsFor: 'testing - instance creation' stamp: 'fbs 9/4/2012 08:55'!
testEmptyIsEmpty
	self assert: DerivingParser empty isEmpty.! !

!DerivingParserTest methodsFor: 'testing - instance creation' stamp: 'fbs 9/4/2012 08:55'!
testEmptyIsMemoized
	self assert: DerivingParser empty == DerivingParser empty.! !

!DerivingParserTest methodsFor: 'testing - instance creation' stamp: 'fbs 9/4/2012 08:59'!
testEmptyStringIsEmptyString
	self assert: {DerivingParser emptyString} asSet equals: DerivingParser emptyString parseNull.! !

!DerivingParserTest methodsFor: 'testing - instance creation' stamp: 'fbs 9/4/2012 08:55'!
testEmptyStringIsMemoized
	self assert: DerivingParser emptyString == DerivingParser emptyString.! !

TestCase subclass: #DerivingParserTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

DerivingParserTestCase subclass: #CatTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!CatTest methodsFor: 'fixtures' stamp: 'fbs 8/30/2012 16:34'!
sampleParser
	^ $a asParser then: $b asParser.! !

!CatTest methodsFor: 'testing - parsing' stamp: 'fbs 9/11/2012 19:56'!
testCompactionCompactsSubparsers
	| c p l r |
	l := $a asParser.
	r := $b asParser.
	p := (DelegateParser new parser: l) then: (DelegateParser new parser: r).
	c := Compacter value: p.
	self assert: c isCat.
	self assert: c first isDelegate.
	self assert: l equals: c first parser.
	self assert: c second isDelegate.
	self assert: r equals: c second parser.! !

!CatTest methodsFor: 'testing - parsing' stamp: 'fbs 9/10/2012 21:15'!
testCompactionOfCompactParsersIsItself
	| p |
	p := $a asParser then: $b asParser.
	self assert: p == (Compacter value: p).! !

!CatTest methodsFor: 'testing - parsing' stamp: 'fbs 9/10/2012 21:15'!
testCompactionWithSingleElementFirstParseTreeIsRed
	| c p |
	p := (EpsStar treeSet: {$a}) then: $b asParser.
	c := Compacter value: p.
	self assert: c isRed.! !

!CatTest methodsFor: 'testing - parsing' stamp: 'fbs 9/10/2012 21:18'!
testCompactionWithSingleElementSecondParseTreeIsRed
	| c p |
	p := $a asParser then: (EpsStar treeSet: {$b}).
	c := Compacter value: p.
	self assert: c isRed.! !

!CatTest methodsFor: 'testing - deriving' stamp: 'fbs 9/13/2012 09:25'!
testDerivativeFollowsLeibniz
	"(ab)' = a'b + ab'."
	| p d |
	p := 1 asParser then: 2 asParser.
	d := p d: 1.
	self assert: {#(1) asOrderedCollection} asSet equals: d parseNull.
	self assert: {#(1 2) asOrderedCollection} asSet equals: (d d: 2) parseNull.! !

!CatTest methodsFor: 'testing - deriving' stamp: 'fbs 9/5/2012 10:09'!
testDerivativeLeftParserNullable
	| p d r |
	r := 1 asParser.
	p := DerivingParser emptyString then: r.
	d := p d: 1.
	"This isn't a Union because the Union smart constructor removes one of the subparsers."
	self assert: d isUnion.
	self assert: d this isCat.
	self assert: d that isCat.
	self assert: d this first isEpsStar.
"	self assert: d this second "
	self assert: d that isEps.
	self assert: r equals: d that second.! !

!CatTest methodsFor: 'testing - accessing' stamp: 'fbs 8/30/2012 22:17'!
testFirst
	| p p1 p2 |
	p1 := $a asParser.
	p2 := $b asParser.
	p := p1 then: p2.
	self assert: p1 equals: p first.! !

!CatTest methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:37'!
testIsCat
	self assert: self sampleParser isCat.! !

!CatTest methodsFor: 'testing - accessing' stamp: 'fbs 8/30/2012 22:19'!
testSecond
	| p p1 p2 |
	p1 := $a asParser.
	p2 := $b asParser.
	p := p1 then: p2.
	self assert: p2 equals: p second.! !

!CatTest methodsFor: 'testing - accessing' stamp: 'fbs 9/5/2012 19:28'!
testSubParsersReturnsFirstAndSecond
	| p |
	p := $a asParser then: $b asParser.
	self assert: {p first. p second} equals: p subParsers.! !

!CatTest methodsFor: 'testing - instance creation' stamp: 'fbs 8/30/2012 16:34'!
testWithWithCreatesParser
	Cat with: $a asParser with: $b asParser.! !

DerivingParserTestCase subclass: #DelayedParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!DelayedParserTest methodsFor: 'fixtures' stamp: 'fbs 8/30/2012 21:28'!
sampleParser
	^ [1 asParser] asParser.! !

!DelayedParserTest methodsFor: 'testing - instance creation' stamp: 'fbs 9/5/2012 09:38'!
testDelayingADelayedParserIsaNoOp
	| p d |
	p := [1 asParser] asParser.
	d := p delay.
	self assert: p == d.! !

!DelayedParserTest methodsFor: 'testing - parsing' stamp: 'fbs 8/31/2012 11:59'!
testDerivativeIsDerivativeOfSubparser
	self assert: ($a asParser delay d: $b) isEmpty.! !

!DelayedParserTest methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 21:30'!
testForce
	| p sub |
	sub := $a asParser.
	p := sub delay.
	self assert: p force == sub.! !

!DelayedParserTest methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 21:29'!
testIsDelayed
	self assert: self sampleParser isDelayed.! !

!DelayedParserTest methodsFor: 'testing - parsing' stamp: 'fbs 8/31/2012 08:07'!
testIsNullable
	"A delay is as nullable as its subparser."
	self assert: ((EpsStar treeSet: {1}) delay) isNullable.
	self deny: (DerivingParser empty delay) isNullable.! !

!DelayedParserTest methodsFor: 'testing - parsing' stamp: 'fbs 8/31/2012 12:00'!
testParseNullIsThatOfSubparser
	self assert: #(1 2) asSet equals: (EpsStar treeSet: #(1 2)) delay parseNull.! !

!DelayedParserTest methodsFor: 'testing - accessing' stamp: 'fbs 9/5/2012 19:29'!
testSubParsersReturnsForced
	| p |
	p := $a asParser delay.
	self assert: {p parser} equals: p subParsers.! !

DerivingParserTestCase subclass: #DelegateParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!DelegateParserTest methodsFor: 'fixtures' stamp: 'fbs 8/31/2012 12:10'!
sampleParser
	^ DelegateParser new parser: $a asParser.! !

!DelegateParserTest methodsFor: 'testing - parsing' stamp: 'fbs 9/11/2012 20:02'!
testCompactionIsCompactionOfSubparser
	| p d |
	p := DerivingParser empty.
	d := DelegateParser new parser: p.
	self assert: d compact parser isEmpty.! !

!DelegateParserTest methodsFor: 'testing - testing' stamp: 'fbs 8/31/2012 12:12'!
testIsAsNullableAsSubparser
	| p |
	p := DelegateParser new parser: (EpsStar treeSet: {1}); yourself.
	self assert: p isNullable.! !

!DelegateParserTest methodsFor: 'testing - testing' stamp: 'fbs 9/11/2012 19:55'!
testIsDelegate
	self assert: self sampleParser isDelegate.! !

!DelegateParserTest methodsFor: 'testing - testing' stamp: 'fbs 8/31/2012 12:11'!
testIsNullable
	| p |
	p := DelegateParser new.
	p parser: $a asParser.
	self deny: p isNullable.! !

!DelegateParserTest methodsFor: 'testing - parsing' stamp: 'fbs 8/31/2012 12:14'!
testParseNullIsThatOfSubparser
	| p |
	p := DelegateParser new
		parser: ((EpsStar treeSet: {1}) or: (EpsStar treeSet: {2}));
		yourself.
	self assert: #(1 2) asSet equals: p parseNull.! !

!DelegateParserTest methodsFor: 'testing - accessing' stamp: 'fbs 8/31/2012 12:09'!
testParser
	| p t |
	t := $a asParser.
	p := DelegateParser new.
	p parser: t.
	self assert: p parser == t.! !

!DelegateParserTest methodsFor: 'testing - testing' stamp: 'fbs 9/12/2012 09:33'!
testResolvedDelegateIsResolved
	self assert: (DelegateParser new parser: $a asParser) isResolved.! !

!DelegateParserTest methodsFor: 'testing - accessing' stamp: 'fbs 9/12/2012 09:27'!
testSubParsersOfUnresolvedDelegateIsEmpty
	| p |
	p := DelegateParser new.
	self assert:#() equals: p subParsers.! !

!DelegateParserTest methodsFor: 'testing - accessing' stamp: 'fbs 9/5/2012 19:30'!
testSubParsersReturnsDelegatedParser
	| p |
	p := DelegateParser new.
	p parser: $a asParser.
	self assert: {p parser} equals: p subParsers.! !

!DelegateParserTest methodsFor: 'testing - testing' stamp: 'fbs 9/12/2012 09:32'!
testUnresolvedDelegateIsNotResolved
	self deny: DelegateParser new isResolved.! !

!DerivingParserTestCase class methodsFor: 'as yet unclassified' stamp: 'fbs 8/30/2012 16:06'!
isAbstract
	^ self name = #DerivingParserTestCase! !

!DerivingParserTestCase methodsFor: 'fixtures' stamp: 'fbs 8/30/2012 16:04'!
sampleParser
	self subclassResponsibility.! !

!DerivingParserTestCase methodsFor: 'testing - parsing' stamp: 'fbs 8/30/2012 18:14'!
testAnythingOrEmptyIsItself
	| p |
	p := self sampleParser.
	self assert: (p or: DerivingParser empty) == p.! !

!DerivingParserTestCase methodsFor: 'testing - parsing' stamp: 'fbs 8/31/2012 09:23'!
testAnythingThenEmptyIsEmpty
	| p |
	p := self sampleParser.
	self assert: (p then: DerivingParser empty) isEmpty.! !

!DerivingParserTestCase methodsFor: 'testing - parsing' stamp: 'fbs 9/11/2012 09:29'!
testCompact
	| p |
	"Most parsers are already compact."
	p := self sampleParser.
	self assert: p equals: (Compacter value: p).! !

!DerivingParserTestCase methodsFor: 'testing - parsing' stamp: 'fbs 8/30/2012 18:16'!
testEmptyOrAnythingIsItself
	| p |
	p := self sampleParser.
	self assert: (DerivingParser empty or: p) == p.! !

!DerivingParserTestCase methodsFor: 'testing - parsing' stamp: 'fbs 8/31/2012 09:23'!
testEmptyThenAnythingIsEmpty
	| p |
	p := self sampleParser.
	self assert: (DerivingParser empty then: p) isEmpty.! !

!DerivingParserTestCase methodsFor: 'testing - parsing' stamp: 'fbs 8/30/2012 17:33'!
testForce
	| p |
	p := self sampleParser.
	self assert: p force == p.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:36'!
testIsCat
	self deny: self sampleParser isCat.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 21:14'!
testIsDelayed
	self deny: self sampleParser isDelayed.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 9/11/2012 19:55'!
testIsDelegate
	self deny: self sampleParser isDelegate.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:03'!
testIsEmpty
	self deny: self sampleParser isEmpty.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:03'!
testIsEps
	self deny: self sampleParser isEps.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 9/4/2012 09:04'!
testIsEpsStar
	self deny: self sampleParser isEpsStar.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:29'!
testIsLazy
	self deny: self sampleParser isLazy.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:13'!
testIsLiteral
	self deny: self sampleParser isLiteral.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 22:12'!
testIsNullable
	self deny: self sampleParser isNullable.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:43'!
testIsRed
	self deny: self sampleParser isRed.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 9/13/2012 19:34'!
testIsStar
	self deny: self sampleParser isStar.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:48'!
testIsUnion
	self deny: self sampleParser isUnion.! !

DerivingParserTestCase subclass: #EmptyStringTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!EmptyStringTest methodsFor: 'fixtures' stamp: 'fbs 9/4/2012 09:01'!
sampleParser
	^ DerivingParser emptyString.! !

!EmptyStringTest methodsFor: 'testing - testing' stamp: 'fbs 9/4/2012 09:02'!
testIsEps
	self assert: self sampleParser isEps.! !

!EmptyStringTest methodsFor: 'testing - testing' stamp: 'fbs 9/4/2012 09:02'!
testIsNullable
	self assert: self sampleParser isNullable.! !

!EmptyStringTest methodsFor: 'testing - parsing' stamp: 'fbs 9/4/2012 09:03'!
testParseNullIsSetWithItself
	"I don't understand this - it seems to just pollute the parse tree set - but it's what dparse.rkt does."
	self assert: {DerivingParser emptyString} asSet equals: DerivingParser emptyString parseNull.! !

!EmptyStringTest methodsFor: 'testing - accessing' stamp: 'fbs 9/5/2012 19:32'!
testSubParsersReturnsEmptySet
	self assert: {} equals: DerivingParser emptyString subParsers.! !

DerivingParserTestCase subclass: #EmptyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!EmptyTest methodsFor: 'fixtures' stamp: 'fbs 8/30/2012 16:05'!
sampleParser
	^ DerivingParser empty.! !

!EmptyTest methodsFor: 'testing - deriving' stamp: 'fbs 8/30/2012 16:11'!
testDerivativeIsItself
	self assert: (DerivingParser empty d: $a) isEmpty.
	self assert: (DerivingParser empty d: $a) == DerivingParser empty.! !

!EmptyTest methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:05'!
testIsEmpty
	self assert: self sampleParser isEmpty.! !

!EmptyTest methodsFor: 'testing - parsing' stamp: 'fbs 8/30/2012 16:17'!
testParseNullIsEmptySet
	self assert: Set new equals: DerivingParser empty parseNull.! !

!EmptyTest methodsFor: 'testing - accessing' stamp: 'fbs 9/5/2012 19:32'!
testSubParsersReturnsEmptySet
	self assert: {} equals: DerivingParser empty subParsers.! !

DerivingParserTestCase subclass: #EpsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!EpsTest methodsFor: 'fixtures' stamp: 'fbs 8/30/2012 16:23'!
sampleParser
	^ EpsStar treeSet: {$a}.! !

!EpsTest methodsFor: 'testing - deriving' stamp: 'fbs 9/4/2012 09:19'!
testDerivativeIsItself
	self assert: ((EpsStar treeSet: {}) d: $a) isEpsStar.
	self assert: Set new equals: ((EpsStar treeSet: {}) d: $a) treeSet.
	self assert: {1. 2} asSet equals: ((EpsStar treeSet: {1. 2}) d: $a) treeSet.! !

!EpsTest methodsFor: 'testing - testing' stamp: 'fbs 9/4/2012 09:04'!
testIsEpsStar
	self assert: self sampleParser isEpsStar.! !

!EpsTest methodsFor: 'testing - testing' stamp: 'fbs 8/31/2012 08:05'!
testIsNullable
	self assert: self sampleParser isNullable.! !

!EpsTest methodsFor: 'testing - parsing' stamp: 'fbs 8/30/2012 16:23'!
testParseNullIsTreeSet
	self assert: {1. 2} asSet equals: (EpsStar treeSet: {1. 2}) parseNull.! !

!EpsTest methodsFor: 'testing - accessing' stamp: 'fbs 9/5/2012 19:32'!
testSubParsersReturnsEmptySet
	self assert: {} equals: (EpsStar treeSet: {1}) subParsers.! !

DerivingParserTestCase subclass: #LiteralTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!LiteralTest methodsFor: 'fixtures' stamp: 'fbs 8/30/2012 16:06'!
sampleParser
	^ $a asParser.! !

!LiteralTest methodsFor: 'testing - deriving' stamp: 'fbs 8/30/2012 16:00'!
testDerivativeOfLiteralWrtAnotherObjectIsEmpty
	| d |
	d := $a asParser d: $b.
	self assert: d isEmpty.! !

!LiteralTest methodsFor: 'testing - deriving' stamp: 'fbs 9/4/2012 09:18'!
testDerivativeOfLiteralWrtSelfIsEps
	| d |
	d := $a asParser d: $a.
	self assert: d isEpsStar.
	self assert: {$a} asSet equals: d treeSet.! !

!LiteralTest methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:13'!
testIsLiteral
	self assert: self sampleParser isLiteral.! !

!LiteralTest methodsFor: 'testing - parsing' stamp: 'fbs 8/30/2012 16:39'!
testParseNullIsEmptySet
	self assert: Set new equals: $a asParser parseNull.! !

!LiteralTest methodsFor: 'testing - accessing' stamp: 'fbs 9/5/2012 19:32'!
testSubParsersReturnsEmptySet
	self assert: {} equals: $a asParser subParsers.! !

DerivingParserTestCase subclass: #RedTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!RedTest methodsFor: 'fixtures' stamp: 'fbs 8/30/2012 16:44'!
sampleParser
	^ (EpsStar treeSet: {1. 2}) reduce: [:x | x + 1].! !

!RedTest methodsFor: 'testing - parsing' stamp: 'fbs 9/17/2012 18:56'!
testCompactOfNestedRedsComposesTheirReducers
	| c p1 p2 |
	p1 := 1 asParser reduce: [:i | i + 1].
	p2 := p1 reduce: [:j | j * 2].
	c := Compacter value: p2.
	self assert: c isRed.
	self assert: c parser isLiteral.
	self assert: 4 equals: (c reducer value: 1).! !

!RedTest methodsFor: 'testing - parsing' stamp: 'fbs 9/18/2012 09:30'!
testCompactionIsRedOfCompactedSubparser
	| p c |
	p := ($a asParser star reduce: [:x | {$b}, x]) derive: $a.
	c := Compacter value: p.
	self assert: c isRed.
	self assert: c parser isStar.
	self assert: $a equals: c parser parser token.! !

!RedTest methodsFor: 'testing - deriving' stamp: 'fbs 8/30/2012 20:42'!
testDerivativeIsTheReductionOfTheSubParsersDerivative
	| d p sub |
	sub := 1 asParser.
	p := sub reduce: [:x | x + 1].
	d := p d: 1.
	self assert: d isRed.
	self assert: {2} asSet equals: d parseNull description: 'The subparser of the derivative is the derivative of the subparser'.
	self assert: d reducer == p reducer description: 'The reducer remains the same'.! !

!RedTest methodsFor: 'testing - testing' stamp: 'fbs 8/31/2012 08:07'!
testIsNullable
	"A reduction is as nullable as its subparser."
	self assert: ((EpsStar treeSet: {1}) reduce: [:x | x + 1]) isNullable.
	self deny: (DerivingParser empty reduce: [:x | x + 1]) isNullable.! !

!RedTest methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:43'!
testIsRed
	self assert: self sampleParser isRed.! !

!RedTest methodsFor: 'testing - accessing' stamp: 'fbs 8/30/2012 22:11'!
testParser
	| p sub red |
	sub := $1 asParser.
	red := [:x | x + 1].
	p := sub reduce: red.
	self assert: sub equals: p parser.
	self assert: red equals: p reducer.! !

!RedTest methodsFor: 'testing - parsing' stamp: 'fbs 9/18/2012 09:38'!
testReduceOfaRedComposesReducers
	| r p |
	p := 1 asParser reduce: [:x | x + 1].
	r := p reduce: [:x | x * 2].
	self assert: r isRed.
	self assert: r parser isLiteral.
	self assert: 4 equals: (r reducer value: 1).! !

!RedTest methodsFor: 'testing - accessing' stamp: 'fbs 8/30/2012 22:11'!
testReducer
	| p sub red |
	sub := $1 asParser.
	red := [:x | x + 1].
	p := sub reduce: red.
	self assert: red equals: p reducer.! !

!RedTest methodsFor: 'testing - accessing' stamp: 'fbs 9/5/2012 19:33'!
testSubParsersReturnsSubParser
	| p |
	p := $a asParser reduce: #yourself.
	self assert: {p parser} equals: p subParsers.! !

DerivingParserTestCase subclass: #StarParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!StarParserTest methodsFor: 'fixtures' stamp: 'fbs 9/13/2012 19:34'!
sampleParser
	^ $a asParser star.! !

!StarParserTest methodsFor: 'testing - parsing' stamp: 'fbs 9/13/2012 20:41'!
testDerivativeOfStarParserIsStarOfDerivativeOfSubParser
	| p d |
	p := $a asParser star.
	d := p d: $a.
	self assert: d isCat.
	self assert: d first isEpsStar.
	self assert: p equals: d second.! !

!StarParserTest methodsFor: 'testing - testing' stamp: 'fbs 9/13/2012 19:34'!
testIsStar
	self assert: self sampleParser isStar.! !

DerivingParserTestCase subclass: #UnionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!UnionTest methodsFor: 'fixtures' stamp: 'fbs 8/30/2012 16:52'!
sampleParser
	^ $a asParser or: $b asParser.! !

!UnionTest methodsFor: 'testing - parsing' stamp: 'fbs 9/5/2012 10:00'!
testDerivativeIsUnionOfSubparserDerivatives
	| d |
	"This strange-looking grammar ensures that the derivative of both subparsers are not isEmpty."
	d := (($a asParser then: $c asParser) or: ($a asParser then: $b asParser)) d: $a.
	self assert: d isUnion.
	"Left subparser is now 'we have read a $a and expect a $c next."
	self assert: d left isCat.
	self assert: d left first isEpsStar.
	self assert: d left second isLiteral.
	self assert: $c equals: d left second token.
	
	"Right subparser is now 'we have read a $a and expect a $b next."
	self assert: d right isCat.
	self assert: d right first isEpsStar.
	self assert: d right second isLiteral.
	self assert: $b equals: d right second token.! !

!UnionTest methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:51'!
testIsUnion
	self assert: self sampleParser isUnion.! !

!UnionTest methodsFor: 'testing - accessing' stamp: 'fbs 8/31/2012 08:46'!
testLeft
	| p p1 p2 |
	p1 := $a asParser.
	p2 := $b asParser.
	p := p1 or: p2.
	self assert: p1 equals: p left.! !

!UnionTest methodsFor: 'testing - parsing' stamp: 'fbs 8/30/2012 16:55'!
testParseNullIsUnionOfSubParsersParseTrees
	self assert: {1. 2} asSet equals: ((EpsStar treeSet: {1}) or: (EpsStar treeSet: {2})) parseNull.! !

!UnionTest methodsFor: 'testing - accessing' stamp: 'fbs 8/31/2012 08:47'!
testRight
	| p p1 p2 |
	p1 := $a asParser.
	p2 := $b asParser.
	p := p1 or: p2.
	self assert: p2 equals: p right.! !

!UnionTest methodsFor: 'testing - accessing' stamp: 'fbs 9/5/2012 19:27'!
testSubParsersReturnsLeftAndRight
	| p |
	p := $a asParser or: $b asParser.
	self assert: {p left. p right} equals: p subParsers.! !

!UnionTest methodsFor: 'testing - parsing' stamp: 'fbs 9/12/2012 09:12'!
testUnionOfCompactedParsersCompactsToItself
	| u |
	u := $a asParser or: $b asParser.
	self assert: u == (Compacter value: u).! !

TestCase subclass: #LeastFixedPointTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!LeastFixedPointTest methodsFor: 'testing - lfp' stamp: 'fbs 8/9/2012 17:29'!
testLfpMemoises
	| f run |
	run := false.
	f := [:x | run := true. x*x - 2] lfpWithBottom: 0.
	f value: 0.
	run := false. "Reset the flag, because the block _will_ have been run in the previous statements."
	f value: 0.
	"The block should not be executed again."
	self deny: run.! !

!LeastFixedPointTest methodsFor: 'testing - lfp' stamp: 'fbs 7/13/2012 18:57'!
testLfpOfIdentityIsOwnThing
	| f |
	f := LeastFixedPoint run: #yourself bottom: 1.
	self assert: 1 equals: (f value: 1).
	self assert: 2 equals: (f value: 2).! !

!LeastFixedPointTest methodsFor: 'testing - lfp' stamp: 'fbs 8/9/2012 17:31'!
testLfpReturnsLeastFixedPoint
	| f |
	"This example doesn't work because lfp really does this: 'run the block until its value stops changing'. Given the simple and pure function below, that's immediately true, and the 'its value stops changing' value is just the first result: f(0) = -2."
	f := [:x | x*x - 2] lfpWithBottom: 0.
	"f(0) = -2. f(-2) = 2. f(2) = 2. So f(f(f(x))) = x when x = 2."
	f value: 0.
	"self assert: 2 equals: fixp."! !

!LeastFixedPointTest methodsFor: 'testing - lfp' stamp: 'fbs 8/30/2012 21:07'!
testLfpWithBottomExplainsCorrectArgCount
	[#+ lfpWithBottom: 0]
		on: ArityMismatch
		do: [:am |
			self assert: 2 equals: am actualArgCount.
			self assert: 1 equals: am expectedArgCount].! !

TestCase subclass: #ParseTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!ParseTests methodsFor: 'testing - parsers' stamp: 'fbs 8/30/2012 16:49'!
testOrCreatesAUnionParser
	self assert: ($a asParser or: $b asParser) isUnion.! !

!ParseTests methodsFor: 'testing - parsers' stamp: 'fbs 9/18/2012 15:49'!
testRightRecursiveGrammar
	| ones d Dones DDones|
	d := DelegateParser new.
	ones := DerivingParser emptyString or: (1 asParser then: d).
	d parser: ones.
	Dones := (ones d: 1) compact.
	DDones := (Dones d: 1) compact.
	self assert: #(#(1)) asSet equals: Dones parseNull.
	self assert: #(#(1 1)) asSet equals: DDones parseNull.! !

!ParseTests methodsFor: 'testing - parsers' stamp: 'fbs 8/30/2012 16:32'!
testThenCreatesACatParser
	self assert: ($a asParser then: $b asParser) isCat.! !

TestCase subclass: #ParserWalkerTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!ParserWalkerTestCase class methodsFor: 'testing' stamp: 'fbs 9/8/2012 23:21'!
isAbstract
	^ self == ParserWalkerTestCase.! !

!ParserWalkerTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 9/8/2012 23:19'!
testParserWalkerVisitsAllSubparsers
	| p |
	p := $a asParser then: ($b asParser or: $c asParser).
	self assert: 5 equals: (self walker walk: p collecting: #yourself) size.! !

!ParserWalkerTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 9/8/2012 23:19'!
testWalkCollectingReturnsMappedValues
	| map p |
	p := $a asParser then: ($b asParser or: $c asParser).
	map := self walker walk: p collecting: #className.
	self assert: #('Union' 'Cat' 'Literal' 'Literal' 'Literal') sorted equals: map sorted.! !

!ParserWalkerTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 9/8/2012 23:19'!
testWalkIsCycleSafe
	| p d |
	d := DelegateParser new.
	p := ($a asParser then: d) or: DerivingParser emptyString.
	d parser: p.
	self assert: 5 equals: (self walker walk: p collecting: #yourself) size.! !

!ParserWalkerTestCase methodsFor: 'fixtures' stamp: 'fbs 9/8/2012 23:19'!
walker
	self subclassResponsibility.! !

ParserWalkerTestCase subclass: #PostOrderParserWalkerTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!PostOrderParserWalkerTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 9/7/2012 17:39'!
testParserWalkerUsesPostorderTraversal
	| p |
	p := $a asParser then: ($b asParser or: $c asParser).
	self assert: #('Literal' 'Literal' 'Literal' 'Union' 'Cat') equals: (PostOrderParserWalker walk: p collecting: #className).! !

!PostOrderParserWalkerTestCase methodsFor: 'fixtures' stamp: 'fbs 9/8/2012 23:19'!
walker
	^ PostOrderParserWalker.! !

ParserWalkerTestCase subclass: #PreOrderParserWalkerTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!PreOrderParserWalkerTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 9/7/2012 17:36'!
testParserWalkerUsesPreorderTraversal
	| p |
	p := $a asParser then: ($b asParser or: $c asParser).
	self assert: #('Cat' 'Literal' 'Union' 'Literal' 'Literal') equals: (ParserWalker walk: p collecting: #className).! !

!PreOrderParserWalkerTestCase methodsFor: 'fixtures' stamp: 'fbs 9/8/2012 23:20'!
walker
	^ PreOrderParserWalker.! !

TestCase subclass: #ParsingBlockClosureExtensionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!ParsingBlockClosureExtensionTest methodsFor: 'helpers' stamp: 'fbs 7/17/2012 20:23'!
assert: aBlock caches: aSelector withArguments: someObjects
	| b run |
	run := false.
	b := [:x | run := true. aBlock value: x] memoized.
	b perform: aSelector withArguments: someObjects.
	run := false.
	b perform: aSelector withArguments: someObjects.
	self deny: run.
	b perform: aSelector withArguments: (someObjects collect: [:x | x + 1]).
	self assert: run.! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing' stamp: 'fbs 8/30/2012 21:08'!
testArityMatchesNumArgs
	self assert: 0 equals: [] arity.
	self assert: 1 equals: [:x | x] arity.
	self assert: 2 equals: [:x :y | x + y] arity.
	self assert: 4 equals: [:x :y :z :w | x + y + z + w] arity.! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing' stamp: 'fbs 8/30/2012 21:13'!
testAsParserReturnsDelayedParser
	self assert: [1] asParser isDelayed.! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing' stamp: 'fbs 7/20/2012 22:05'!
testLfpRequiresUnaryBlock
	self should: [[1] lfpWithBottom: 0] raise: ArityMismatch.! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing' stamp: 'fbs 7/13/2012 18:50'!
testLfpReturnsALeastFixedPoint
	self assert: LeastFixedPoint equals: ([:x | x ] lfpWithBottom: 0) class.! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing - memoizing' stamp: 'fbs 7/17/2012 20:20'!
testMemoizedBlockCachesResults
	self assert: [:x | x ] caches: #value: withArguments: #(1).! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing - memoizing' stamp: 'fbs 7/17/2012 20:22'!
testMemoizedBlockCachesResultsFromValueValue
	| b run |
	b := [:x :y | run := true. x + y] memoized.
	b value: 1 value: 2.
	run := false.
	b value: 1 value: 2.
	self deny: run.
	b value: 1 value: 1.
	self assert: run.! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing - memoizing' stamp: 'fbs 7/17/2012 20:22'!
testMemoizedBlockCachesResultsFromValueValueValue
	| b run |
	b := [:x :y :z | run := true. x + y + z] memoized.
	b value: 1 value: 2 value: 3.
	run := false.
	b value: 1 value: 2 value: 3.
	self deny: run.
	b value: 1 value: 2 value: 1.
	self assert: run.! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing - memoizing' stamp: 'fbs 7/17/2012 20:22'!
testMemoizedBlockCachesResultsFromValueValueValueValue
	| b run |
	b := [:x :y :z :w | run := true. x + y + z + w] memoized.
	b value: 1 value: 2 value: 3 value: 4.
	run := false.
	b value: 1 value: 2 value: 3 value: 4.
	self deny: run.
	b value: 1 value: 2 value: 3 value: 1.
	self assert: run.! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing - memoizing' stamp: 'fbs 7/17/2012 18:46'!
testMemoizedBlockReturnsExpectedValue
	| b |
	b := [:x | x] memoized.

	self assert: 1 equals: (b value: 1).
	self assert: 2 equals: (b value: 2)! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing - memoizing' stamp: 'fbs 7/17/2012 18:44'!
testMemoizedReturnsMemoizingBlock
	self assert: ([:x | x] memoized isKindOf: MemoizingBlock).! !

TestCase subclass: #ParsingCollectionExtensionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!ParsingCollectionExtensionTest methodsFor: 'as yet unclassified' stamp: 'fbs 9/5/2012 09:12'!
testCartesianProductWithEmptyLeftReturnsEmptyCollection
	self assert: Set new equals: (Set new cartesianProduct: #+ with: #(1) asSet).
	self assert: #() equals: (#() cartesianProduct: #+ with: #(1)).
	self assert: OrderedCollection new equals: (OrderedCollection new cartesianProduct: #+ with: (OrderedCollection with: 1)).! !

!ParsingCollectionExtensionTest methodsFor: 'as yet unclassified' stamp: 'fbs 9/5/2012 09:13'!
testCartesianProductWithEmptyRightReturnsEmptyCollection
	self assert: Set new equals: (#(1) asSet cartesianProduct: #+ with: Set new).
	self assert: #() equals: (#(1) cartesianProduct: #+ with: #()).
	self assert: OrderedCollection new equals: ((OrderedCollection with: 1) cartesianProduct: #+ with: OrderedCollection new).! !

!ParsingCollectionExtensionTest methodsFor: 'as yet unclassified' stamp: 'fbs 9/5/2012 09:16'!
testCartesianProductWithNonEmptyCollections
	"We show the duplicate 5 to explain the workings of the product. As a Set, the duplicate will be removed."
	self assert: #(4 5 5 6) asSet equals: (#(1 2) asSet cartesianProduct: #+ with: #(3 4) asSet).
	self assert: #(4 5 5 6) equals: (#(1 2) cartesianProduct: #+ with: #(3 4)).
	self assert: #(4 5 5 6) asOrderedCollection equals: (#(1 2) asOrderedCollection cartesianProduct: #+ with: #(3 4) asOrderedCollection).! !

TestCase subclass: #ParsingObjectExtensionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!ParsingObjectExtensionTest methodsFor: 'test - instance creation' stamp: 'fbs 8/30/2012 16:07'!
testAsParserReturnsLiteral
	self assert: $a asParser isLiteral.
	self assert: 1 asParser isLiteral.
	self assert: Object new asParser isLiteral.! !

TestCase subclass: #ParsingSymbolExtensionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!ParsingSymbolExtensionTest methodsFor: 'as yet unclassified' stamp: 'fbs 8/30/2012 21:06'!
testLfpRequiresUnarySymbol
	self should: [#+ lfpWithBottom: 0] raise: ArityMismatch.! !
DerivingParser initialize!
PostOrderParserWalker initialize!
